# -*- coding: utf-8 -*-
"""
file: themis_gmag.py

Desrciption:  
    Data loading functions for the THEMIS GMAG stations
    
Example:
    pyspedas.themis_load_gmag(['2015-12-31'], ['bmls', 'ccnv', 'drby'])
    
Parameters:
    dates: str/list of str ['yyyy-mm-dd']
        List of dates to be downloaded (eg. ['2015-12-31'])
    stations: str/list of str
        GMAG station names
    downloadonly: bool (True/False)
        If True then CDF files are downloaded only, if False then they are also loaded into pytplot using pytplot.cdf_to_tplot
    varformat : str
        The file variable formats to load into tplot.  Wildcard character 
            "*" is accepted.  By default, all variables are loaded in.  
    get_support_data: bool
        Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".
    prefix: str
        The tplot variable names will be given this prefix.  By default, no prefix is added.
    suffix: str
        The tplot variable names will be given this suffix.  By default, no suffix is added.
        
"""

import requests
import pytplot
import os
import pyspedas

gmag_dict = {}

def query_gmags():
    """ returns a dictionary of gmag stations and all their metadata        
    """
    
    url = 'http://themis.ssl.berkeley.edu/gmag/gmag_json.php'
    global gmag_dict
    
    params = dict(
        station='',
        group=''
    )    
    resp = requests.get(url=url, params=params)
    data = resp.json() 
    gmag_dict = data
    
    return data

def get_group(station_name):
    """ returns a list with the groups that station belongs to
    """
    
    global gmag_dict
    group_list = []
    if gmag_dict=={}:
        gmag_dict = query_gmags()
        
    for station in gmag_dict:
        if station['ccode'].lower() == station_name:
            for key, value in station.items():
                if value == 'Y':
                    if station['ccode'].lower() not in group_list:
                         group_list.append(key)
    
    return group_list

def gmag_list(group='all'):
    """ returns a list of stations
        prints a list of "stations: start date - end date"
    """
    
    global gmag_dict
    station_list = []
    if gmag_dict=={}:
        gmag_dict = query_gmags()
        
    for station in gmag_dict:
        station_name = station['ccode'].lower()
        station_group = get_group(station_name)
        if group == 'all' or group == '*' or group == '' or group in station_group:
            station_list.append(station_name)
            print(station_name + ": from " + station['day_first'] + " to " + station['day_last'])
    
    return station_list

def gmag_groups():
    """ returns a dictionary of station groups with a list of stations "group:[stations]"
        prints a list of "group:'stations'"
    """
    
    global gmag_dict
    group_dict = {}
    if gmag_dict=={}:
        gmag_dict = query_gmags()
        
    for station in gmag_dict:
        for key, value in station.items():
            if value == 'Y':
                if key in group_dict:
                    if station['ccode'].lower() not in group_dict[key]:
                     group_dict[key].append(station['ccode'].lower())
                else:
                    group_dict[key] = []
                    group_dict[key].append(station['ccode'].lower())

    
    #print them
    print()
    for g, s in group_dict.items():
        print(g + ":" + ",'".join(s) +"'")
    
    return group_dict
 

def check_gmag(station_name):
    """ returns 1 if station_name is in the gmag list, 0 otherwise"
    """  
    
    global gmag_dict
    if gmag_dict=={}:
        gmag_dict = query_gmags()
        
    for station in gmag_dict:
        if station['ccode'].lower() == station_name.lower():
            return 1             
    
    return 0

def check_greenland(station_name):
    """ returns 1 if station_name is in the greenland gmag list, 0 otherwise"
    """    
    global gmag_dict
    if gmag_dict=={}:
        gmag_dict = query_gmags()
        
    for station in gmag_dict:
        if station['ccode'].lower() == station_name.lower():
            if (station['country'] != None and station['country'].lower() == 'greenland') or (station['greenland'] != None and station['greenland'].lower() == 'y'):
                return 1             
    
    return 0
    
def gmag_filename(dates, stations):
    """Create a list of tuples for downloading: remote_file, local_file"""
    prefs = pyspedas.get_spedas_prefs()
    if 'themis_remote' in prefs: 
        remote_path = prefs['themis_remote']
    else:
        raise NameError('remote_path is not found in spd_prefs.txt')
    if 'data_dir' in prefs: 
        data_dir = prefs['data_dir']
        
    else:
        raise NameError('data_dir is not found in spd_prefs.txt')

    dates = pyspedas.get_dates(dates)
    
    file_list = []
    
    probe = 'thg'
    level = 'l2'
    instrument = 'mag'
    version = '1'
    if stations[0] == 'idx':
        level = 'l1'
    
    for sdate in dates:
        year = sdate[0:4]
        month = sdate[5:7]
        day = sdate[8:10]
        for station in stations: 
            # file_dir = 'tha/l2/fgm/2015/'
            if station == 'idx':
                level = 'l1'
                file_dir = probe + '/' + level + '/' + instrument + '/' + station + '/' + year
                filename = probe + '_' + level + '_' + station + '_' + year + month + day + '_v0' + version + '.cdf'              
            elif check_greenland(station):
                #thg/greenland_gmag/l2
                file_dir = probe + '/greenland_gmag/' + level + '/' + station + '/' + year
                filename = probe + '_' + level + '_' + instrument + '_' + station + '_' + year + month + day + '_v0' + version + '.cdf'         
            else:
                #thg/l2/mag/
                file_dir = probe + '/' + level + '/' + instrument + '/' + station + '/' + year
                filename = probe + '_' + level + '_' + instrument + '_' + station + '_' + year + month + day + '_v0' + version + '.cdf'
            
            
            file_dir_local = os.path.join(probe, level, instrument, station, year)
            # thg_l2_mag_amd_20170109_v01.cdf	
            
            remote_file = remote_path + file_dir  + '/' + filename
            local_file = os.path.join(data_dir, file_dir_local, filename)
            
            file_list.append((remote_file, local_file))                

    return file_list
    
def themis_load_gmag(dates, stations, downloadonly=False, varformat=None, get_support_data=False, prefix='', suffix=''):
    """Loads themis data into pytplot variables"""
    file_list = gmag_filename(dates, stations)
    
    count = 0
    dcount = 0
    for remotef, localf in file_list:   
        count += 1
        resp, err, locafile = pyspedas.download_files(remotef, localf)
        if resp:
            print(str(count) + '. File was downloaded. Location: ' + locafile)
            dcount += 1
            if not downloadonly:
                try: 
                    testcdf = pytplot.cdf_to_tplot(locafile, varformat, get_support_data, prefix, suffix)                
                    print(testcdf) 
                except TypeError as e:
                    msg = "cdf_to_tplot could not load " + locafile + "\nError:\n" + str(e)
                    print(msg)
            
        else:
            print(str(count) + '. There was a problem. Could not download file: ' + remotef)
            print(err)        

    print('Downloaded ' + str(dcount) + ' files.')
    print('tplot variables:')
    print(pytplot.tplot_names())    
    


# -*- coding: utf-8 -*-
"""
file: themis_load.py

Desrciption:  
    Data loading functions for the THEMIS mission
    
Example:
    pyspedas.themis_load(['2015-12-31'], ['tha'], '*', ['l2'])
    
Parameters:
    dates: str/list of str ['yyyy-mm-dd']
        List of dates to be downloaded (eg. ['2015-12-31'])
    probes: str/list of str
        Probes for missions with multiple probes (eg. ['tha', 'thb']), wildcard ('*') for all probes
    instruments: str/list of str
        List of instruments (eg. ['fft']), wildcard ('*') for all instruments
    level: str
        Usually, either 'l2' or 'l1'
    downloadonly: bool (True/False)
        If True then CDF files are downloaded only, if False then they are also loaded into pytplot using pytplot.cdf_to_tplot
    varformat : str
        The file variable formats to load into tplot.  Wildcard character 
            "*" is accepted.  By default, all variables are loaded in.  
    get_support_data: bool
        Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".
    prefix: str
        The tplot variable names will be given this prefix.  By default, no prefix is added.
    suffix: str
        The tplot variable names will be given this suffix.  By default, no suffix is added.

Notes:      
    List of possible values for "probes" variable:
    ['tha', 'tha', 'thc', 'thd', 'the']
    
    List of possible values for L2 themis "instruments" variable:
    ['efi', 'esa', 'fbk', 'fft', 'fgm', 'fit', 'gmom', 'mom', 'scm', 'sst']
    
    List of possible values for L1 themis "instruments" variable:
    ['bau', 'eff', 'efp', 'efw', 'esa', 'fbk', 'fff_16', 'fff_32', 'fff_64', 'ffp_16', 'ffp_32', 'ffp_64', 'ffw_16', 'ffw_32', 'ffw_64', 'fgm', 'fit', 'hsk', 'mom', 'scf', 'scm', 'scmode', 'scp', 'scw', 'spin', 'sst', 'state', 'trg', 'vaf', 'vap', 'vaw']

"""

import os
import pytplot
import pyspedas 
from .themis_helpers import get_instruments, get_probes

def themis_filename(dates, probes, instruments, level):
    """Create a list of tuples for downloading: remote_file, local_file"""
    prefs = pyspedas.get_spedas_prefs()
    if 'themis_remote' in prefs: 
        remote_path = prefs['themis_remote']
    else:
        raise NameError('remote_path is not found in spd_prefs.txt')
    if 'data_dir' in prefs: 
        data_dir = prefs['data_dir']+'themis'
        
    else:
        raise NameError('data_dir is not found in spd_prefs.txt')

    if level!='l1':
        level = 'l2'    
    probes = get_probes(probes)
    instruments = get_instruments(instruments, level)
    dates = pyspedas.get_dates(dates)

    if level == 'l1':
        version = '?'
    else:
        version = '1'
    
    file_list = []
    for sdate in dates:
        year = sdate[0:4]
        month = sdate[5:7]
        day = sdate[8:10]
        for probe in probes:
            for instrument in instruments: 
                # file_dir = 'tha/l2/fgm/2015/'
                file_dir = probe + '/' + level + '/' + instrument + '/' + year
                file_dir_local = os.path.join(probe, level, instrument, year)
                # filename = 'tha_l2_fgm_20150101_v01.cdf'
                filename = probe + '_' + level + '_' + instrument + '_' + year + month + day + '_v0' + version + '.cdf'
                
                remote_file = remote_path + '/' + file_dir  + '/' + filename
                local_file = os.path.join(data_dir, file_dir_local, filename)
                
                file_list.append((remote_file, local_file))                

    return file_list


def themis_load(dates, probes, instruments, level, downloadonly=False, varformat=None, get_support_data=False, prefix='', suffix=''):
    """Loads themis data into pytplot variables"""
    
    file_list = themis_filename(dates, probes, instruments, level)
    
    count = 0
    dcount = 0
    for remotef, localf in file_list:   
        count += 1
        resp, err, locafile = pyspedas.download_files(remotef, localf)
        if resp:
            print(str(count) + '. File was downloaded. Location: ' + locafile)
            dcount += 1
            if not downloadonly:
                try: 
                    testcdf = pytplot.cdf_to_tplot(locafile, varformat, get_support_data, prefix, suffix)                
                    print(testcdf) 
                except TypeError as e:
                    msg = "cdf_to_tplot could not load " + locafile + "\nError:\n" + str(e)
                    print(msg)
            
        else:
            print(str(count) + '. There was a problem. Could not download file: ' + remotef)
            print(err)        

    print('Downloaded ' + str(dcount) + ' files.')
    print('tplot variables:')
    print(pytplot.tplot_names())


# -*- coding: utf-8 -*-
"""
file: prefs.py

Desrciption:  
    Load preferences 

"""

import os

def get_prefs_filename():

    dir_path = os.path.abspath(os.path.dirname(__file__))
    fname = os.path.join(dir_path, 'spd_prefs_txt.py')
    
    print(fname)
    
    return fname    


def get_spedas_prefs():
    """Get all the spedas preferences and return a directory"""    
    
    fname = get_prefs_filename()
    print("Preferences file: " + fname )
    
    """Read preferences"""
    with open(fname, 'r') as f:
        content = f.readlines()
    content = [x.strip() for x in content] 
    
    """Fill dictionary""" 
    ans_dict = {}
    for line in content:
        if len(line)>2 and line[0]!="#":
            terms = line.split('=')
            terms = [x.strip() for x in terms]
            if len(terms)==2:
                if terms[0]!='' and terms[1]!='':
                    ans_dict[terms[0].replace("'","")] = terms[1].replace("'","")    
 
    f.closed	
    return ans_dict
 
def set_spedas_prefs(var_name, var_value):
    """Get all the spedas preferences and return a directory""" 

    found = 0
    if var_name == '' or var_value == '':
	    return found
    
    fname = get_prefs_filename()
    print("Preferences file: " + fname )
    
    """Read preferences"""
    with open(fname, 'r') as f:
        content = f.readlines()
    content = [x.strip() for x in content] 
    
    """Make change"""
    new_content = ''
    for line in content:
        new_line = line
        if len(line)>2 and line[0]!="#":
            terms = line.split('=')
            terms = [x.strip() for x in terms]
            if len(terms)==2:
                if terms[0]!='' and terms[1]!='':
                    if terms[0] == var_name:
                        new_line = var_name + "='" + var_value + "'"  
                        found = 1
        new_line = new_line.replace(os.linesep, '\n').strip()
        if new_line!='':		
            new_content += new_line + '\n'
	
    with open(fname, 'w') as f:
        f.write(new_content) 
    f.closed
	
    return found
                       
    


# -*- coding: utf-8 -*-
"""
file: basic_gmag.py

Desrciption:  
    Basic example with THEMIS GMAG data.
    Downloads THEMIS data from EPO GMAG stations and plots it.

"""

import pyspedas
import pytplot  

def basic_gmag():
    # Print the installed version of pyspedas
    pyspedas.get_spedas_prefs() 
    # Delete any existing pytplot variables
    pytplot.del_data()     
    # Define list of dates
    time_list = ['2015-12-31']
    # Get a list of EPO gmag stations
    sites = pyspedas.gmag_list(group='epo')
    #Download gmag files and load into pyspedas variables
    pyspedas.themis_load_gmag(time_list, sites)
    #Download AE index data
    pyspedas.themis_load_gmag(time_list, ['idx'])
    #Get a list of loaded sites
    sites_loaded = pytplot.tplot_names()
    
    #Subtact mean values    
    for station_var in sites_loaded:
        if station_var not in ['thg_idx_al', 'thg_idx_au', 'thg_idx_ae']:
            time, data = pytplot.get_data(station_var)  
            data1 = data - data.mean(axis=0, keepdims=True)
            pytplot.store_data(station_var, data={'x':time, 'y':data1})
    #Plot 
    pytplot.tplot_options('title', 'EPO GMAG 2015-12-31')    
    pytplot.tplot(sites_loaded)

#Run the example code
#basic_gmag()


# -*- coding: utf-8 -*-
"""
file: basic_analysis.py

Desrciption:  
    Basic example using analysis functions.
    Downloads THEMIS data and plots it.

"""

import pyspedas
import pytplot

def basic_analysis():
    # Print the installed version of pyspedas
    pyspedas.spd_version()
    # Delete any existing pytplot variables
    pytplot.del_data()     
    # Download THEMIS state data for 2015-12-31
    pyspedas.load_data('themis', '2015-12-31', ['tha'], 'state', 'l1')
    # Use some analysis functions on tplot variables
    pyspedas.subtract_average('tha_pos', '')
    pyspedas.subtract_median('tha_pos', '')
    # Plot 
    pytplot.tplot(["tha_pos","tha_pos-d","tha_pos-m"])

#Run the example code
#basic_analysis()


# -*- coding: utf-8 -*-
#Locations of remote files and local directory for cdf files
#You can change data_dir to a directory of your choice
data_dir='C:\\Datapy\\'
themis_remote='http://themis.ssl.berkeley.edu/data/themis'
omni_remote='https://cdaweb.gsfc.nasa.gov/istp_public/data/omni'



# -*- coding: utf-8 -*-
"""
file: omni_load.py

Desrciption:  
    Data loading functions for OMNI  
    
Example:
    pyspedas.omni_load('2015-12-31', '5min', False)
    
Parameters:
    dates: str/list of str ['yyyy-mm-dd']
        List of dates to be downloaded (eg. ['2015-12-31'])
    level: str
        Either '1min' or '5min' 
    downloadonly: bool (True/False)
        If True then CDF files are downloaded only, if False then they are also loaded into pytplot using pytplot.cdf_to_tplot
    varformat : str
        The file variable formats to load into tplot.  Wildcard character 
            "*" is accepted.  By default, all variables are loaded in.  
    get_support_data: bool
        Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".
    prefix: str
        The tplot variable names will be given this prefix.  By default, no prefix is added.
    suffix: str
        The tplot variable names will be given this suffix.  By default, no suffix is added.

"""

import os
import pytplot
import pyspedas 

def omni_filename(dates, level):
    """Create a list of tuples for downloading: remote_file, local_file"""
    prefs = pyspedas.get_spedas_prefs()
    if 'omni_remote' in prefs: 
        #https://cdaweb.gsfc.nasa.gov/istp_public/data/omni/hro_1min/2017/
        remote_path = prefs['omni_remote']
    else:
        raise NameError('remote_path is not found in spd_prefs.txt')
    if 'data_dir' in prefs: 
        data_dir = prefs['data_dir'] + 'omni'
        
    else:
        raise NameError('data_dir is not found in spd_prefs.txt')

    if level!='1min':
        level = '5min'    
        
    dates = pyspedas.get_dates(dates)
    version = '?'
    
    file_list = []
    for sdate in dates:
        year = sdate[0:4]
        month = sdate[5:7]
        day = sdate[8:10]
        
        # file_dir = 'hro_1min/2017/'
        file_dir = 'hro_' + level + '/' + year 
        file_dir_local = os.path.join('hro_', level, year)
        # filename = 'omni_hro_1min_20171201_v01.cdf'
        filename = 'omni_hro_' + level + '_' + year + month + '01' + '_v0' + version + '.cdf'
        
        remote_file = remote_path + '/' + file_dir  + '/' + filename
        local_file = os.path.join(data_dir, file_dir_local, filename)
        
        file_list.append((remote_file, local_file))                

    return file_list


def omni_load(dates, level, downloadonly=False, varformat=None, get_support_data=False, prefix='', suffix=''):
    """Loads themis data into pytplot variables"""
    
    file_list = omni_filename(dates, level)
    
    count = 0
    dcount = 0
    for remotef, localf in file_list:   
        count += 1
        resp, err, locafile = pyspedas.download_files(remotef, localf)
        if resp:
            print(str(count) + '. File was downloaded. Location: ' + locafile)
            dcount += 1
            if not downloadonly:
                try: 
                    testcdf = pytplot.cdf_to_tplot(locafile, varformat, get_support_data, prefix, suffix)                
                    print(testcdf) 
                except TypeError as e:
                    msg = "cdf_to_tplot could not load " + locafile + "\nError:\n" + str(e)
                    print(msg)
            
        else:
            print(str(count) + '. There was a problem. Could not download file: ' + remotef)
            print(err)        

    print('Downloaded ' + str(dcount) + ' files.')
    print('tplot variables:')
    print(pytplot.tplot_names())


from .tnames import tnames
from .load_data import load_data
from .spd_version import spd_version
from .prefs import get_spedas_prefs
from .helpers import download_files, url_exists, find_latest_url_version
from .dates import validate_date, get_date_list, get_dates
from .themis.themis_load import themis_filename, themis_load
from .themis.themis_gmag import themis_load_gmag, gmag_filename, check_gmag, check_greenland, gmag_groups, gmag_list, get_group, query_gmags
from .themis.themis_helpers import get_probes, get_instruments
from .omni.omni_load import omni_filename, omni_load
from .analysis.subtract_average import subtract_average
from .analysis.subtract_median import subtract_median
from .examples.basic_example import basic_example
from .examples.basic_analysis import basic_analysis
from .examples.basic_gmag import basic_gmag
from .examples.basic_spectra import basic_spectra


# -*- coding: utf-8 -*-
"""
file: tnames.py

Desrciption:  
    Finds all tplot names that follow a pattern that contains a wildcard
    Wildcards: ? for a single character, * from multiple characters
    
Returns:
    A list of tplot names
"""

import pytplot
import fnmatch

def tnames(pattern=None):
    all_names = pytplot.tplot_names()
    if pattern==None:
        return all_names
    else:
        filtered = fnmatch.filter(all_names, pattern)
        return filtered 


# -*- coding: utf-8 -*-
"""
file: spd_version.py

Desrciption:  
    Returns the version number of pyspedas

"""

def spd_version(): 
    import os.path
    dir_path = os.path.abspath(os.path.dirname(__file__))
    fname = os.path.join(dir_path, 'spd_prefs_txt.py')
    print("Preferences file: " + fname )     
    
    import pkg_resources
    ver = pkg_resources.get_distribution("pyspedas").version
    print("pyspedas version: " + ver)  


# -*- coding: utf-8 -*-
"""
file: subtract_average.py

Desrciption:  
    Subtract the average from the data.
    Result well be saved in:
        - The same tplot variable names if new_names is not given
        - The new_names list if it has same length as names
        - The name followed by '-d' otherwise
        
    Wildcards in names: ? for a single character, * from multiple characters

"""
import pyspedas
import pytplot
import numpy

def subtract_average(names, new_names=None):
    old_names = pyspedas.tnames(names)
    if len(old_names) < 1:
        print('Subtract Average error: No tplot variables given.')
        return 
    
    if new_names == None:
        n_names = names
    elif new_names == '*' or new_names == '':
        n_names = [s + '-d' for s in old_names]
    else:
        n_names = new_names
        
    if len(n_names) != len(old_names):
        n_names = [s + '-d' for s in old_names]
    
    for i in range(len(old_names)):
        time, data = pytplot.get_data(old_names[i]) 
        new_data = data-numpy.mean(data, axis=0)        
        pytplot.store_data(n_names[i], data={'x':time, 'y':new_data})
        print('Subtract Average applied to: '+ n_names[i])
        
        
    


# -*- coding: utf-8 -*-
"""
file: dates.py

Desrciption:  
    Date functions 

"""

import datetime  

def validate_date(date_text):
    """Validate a date (yyyy-mm-dd)"""
    try:
        datetime.datetime.strptime(date_text, '%Y-%m-%d')
    except ValueError:
        raise ValueError("Incorrect data format, should be yyyy-mm-dd: '" + date_text + "'")

def get_date_list(date_start, date_end):
    """Returns a list of dates between start and end dates"""
    d1 = datetime.date(int(date_start[0:4]), int(date_start[5:7]), int(date_start[9:11]))
    d2 = datetime.date(int(date_end[0:4]), int(date_end[5:7]), int(date_end[9:11]))
  
    delta = d2 - d1        
    ans_list = []
    
    for i in range(delta.days + 1):
        ans_list.append(str(d1 + datetime.timedelta(days=i)))
    
    return ans_list    

def get_dates(dates):
    """Returns a list of dates
       date format: yyyy-mm-dd
       input can be a single date or a list of two (start and end date)
    """    
    ans_list = []
    if not isinstance(dates, (list, tuple)):
        dates = [dates]
        
    if len(dates) == 1:
        try:
            validate_date(dates[0])
            ans_list = dates            
        except ValueError as e:
            print(e)
            ans_list = []        
    elif len(dates) == 2:        
        try:
            validate_date(dates[0])        
            validate_date(dates[1])
            ans_list = get_date_list(dates[0], dates[1])           
        except ValueError as e:
            print(e)
            ans_list = []
      
    return ans_list



# -*- coding: utf-8 -*-
"""
file: basic_example.py

Desrciption:  
    Basic example to verify that the installation works correctly.
    Downloads THEMIS data and plots it.

"""

import pyspedas
import pytplot

def basic_example():
    # Print the installed version of pyspedas
    pyspedas.get_spedas_prefs() 
    # Delete any existing pytplot variables
    pytplot.del_data()     
    # Download THEMIS state data for 2015-12-31
    pyspedas.load_data('themis', '2015-12-31', ['tha'], 'state', 'l1')
    # Get data into python variables
    time, data = pytplot.get_data("tha_pos")
    # Store a new pytplot variable
    pytplot.store_data("tha_position", data={'x':time, 'y':data})
    # Define the y-axis limits
    pytplot.ylim('tha_pos', -23000.0, 81000.0)
    pytplot.ylim('tha_position', -23000.0, 81000.0)
    pytplot.ylim('tha_vel', -8.0, 12.0)
    # Plot position and velocity using the pyqtgraph library (default)
    pytplot.tplot(["tha_pos","tha_position", "tha_vel"])
    # Plot position and velocity using the bokeh library, requires nodejs ("conda install nodejs") 
    pytplot.tplot(["tha_pos","tha_position", "tha_vel"], bokeh=True)

#Run the example code
#basic_example()


# -*- coding: utf-8 -*-
"""
file: load_data.py

Desrciption:  
    Main function for pyspedas

Example:
    pyspedas.load_data('themis', '2015-12-31', ['tha'], '*', 'l2', False)
    
Parameters:
    mission: str/list of str
        The name of the mission (eg. 'themis')
    dates: str/list of str ['yyyy-mm-dd']
        List of dates to be downloaded (eg. ['2015-12-31'])
    probes: str/list of str
        Probes for missions with multiple probes (eg. ['tha', 'thb']), wildcard ('*') for all probes
    instruments: str/list of str
        List of instruments (eg. ['fft']), wildcard ('*') for all instruments
    level: str
        Usually, either 'l2' or 'l1', depends on mission
    downloadonly: bool (True/False)
        If True then CDF files are downloaded only, if False then they are also loaded into pytplot using pytplot.cdf_to_tplot
    varformat : str
        The file variable formats to load into tplot.  Wildcard character 
            "*" is accepted.  By default, all variables are loaded in.  
    get_support_data: bool
        Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".
    prefix: str
        The tplot variable names will be given this prefix.  By default, no prefix is added.
    suffix: str
        The tplot variable names will be given this suffix.  By default, no suffix is added.
"""


def load_data(mission, dates, probes, instruments, level, downloadonly=False, varformat=None, get_support_data=False, prefix='', suffix=''):
    """Loads data from mission into pytplot"""    
    
    if mission == 'themis':
        from pyspedas import themis_load
        themis_load(dates, probes, instruments, level, downloadonly, varformat, get_support_data, prefix, suffix)
    if mission == 'omni':
        from pyspedas import omni_load
        #level = 1m, 5m
        omni_load(dates, level, downloadonly, varformat, get_support_data, prefix, suffix)
    else:
        print("Currently, only the THEMIS and OMNI missions are implemented.")    
    
    print('Data loading finished.')

 



# -*- coding: utf-8 -*-
"""
file: basic_spectra.py

Desrciption:  
    Basic example with THEMIS data.
    Downloads THEMIS data and plots a spectrogram.
    
"""

import pyspedas
import pytplot

def basic_spectra():
    # Print the installed version of pyspedas
    pyspedas.get_spedas_prefs() 
    # Delete any existing pytplot variables
    pytplot.del_data()     
    # Download THEMIS state data for 2015-12-31
    pyspedas.load_data('themis', '2015-12-31', ['tha'], 'sst', 'l2')
    pytplot.tplot_options('title', 'tha_psif_en_eflux 2015-12-31') 
    pytplot.ylim('tha_psif_en_eflux', 10000.0, 10000000.0)
    pytplot.options('tha_psif_en_eflux', 'colormap', 'viridis')
    pytplot.tplot(['tha_psif_en_eflux'])

#Run the example code
#basic_spectra()


# -*- coding: utf-8 -*-
"""
file: helpers.py

Desrciption:  
    Helper functions for pyspedas

"""

import urllib
import os

def download_files(url, locafile):
    """Get a file from internet and save it localy."""
    
    exists = False    
    httpreq = None
    err = None
    ver = -1
    
    exists, err, ver, newurl = find_latest_url_version(url)
    if not exists:
        return exists, err, locafile
    
    url = newurl
    if ver != -1:
        locafile = locafile.replace('?', str(ver))
    
    #Create local directory if it does not exist
    dirPath = os.path.dirname(locafile)
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)
    
    # Download file
    try:
        httpreq = urllib.request.urlretrieve(url, locafile)
    except urllib.request.URLError as e:
        httpreq = None
        err = e        
        
    if httpreq is not None:    
        exists = True
  
    return exists, err, locafile

def url_exists(url):
    """Returns True if url exists, otherwise False"""
    
    ans = False
    try:
        urllib.request.urlopen(url)
        ans = True
        err = ''
    except urllib.request.URLError as e:
        ans = False   
        err = e
        
    return ans, err

def find_latest_url_version(url):
    """Returns if a file exists and the latest version.
    Assumes max version = v04, should be changed if there are higher versions
    """
    
    exists = False
    err = ''
    ver = -1
    newurl = url
    if '?' in url:
        for ver in range(4, -1, -1):
            newurl = url.replace('?', str(ver))
            exists, err = url_exists(newurl)
            #print(exists, err, ver, newurl)
            if exists:
                break
    else:
        ver = -1
        newurl = url
        exists, err = url_exists(newurl)
        return exists, err, ver, newurl
    
    return exists, err, ver, newurl

